/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the
edition of Latex documents

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
* Class LatexElement
* Creation: 14/06/2010
* version 1.0 14/06/2010
* @author Ludovic APVRILLE
* @see 
*/

package latexstructure;

import java.io.*;
import java.util.*;
import javax.swing.ImageIcon;

import myutil.*;
import ui.*;


public	class LatexElement implements GenericTree {
	
	public final static String MAIN_SLIDE= "Main slide";
	public final static ImageIcon MAIN_SLIDE_ICON = IconManager.imgic110;
	
	public final static String []keywords = {"%", "\\setbeamer", "\\author{", "\\bibliography{", "\\begin{abstract}", "\\title{", "\\includegraphics", "\\includegraphics", "\\frame{", "\\frame[", "\\chapter{", "\\chapter*{", "\\section{", "\\subsection{", "\\subsubsection{", "\\label{", 
	"@inproceedings{", "@article{", "@misc{", "@techreport{", "@TechReport{", "@phdthesis{", "@incollection{", "@book{", "@manual"};
	
	// -1 means comment
	
	public final static int [] levels = {-1, 0, 0, 0, 0, 0, 10, 10, 5, 5, 1, 1, 2, 3, 4, 10, 
	0, 0, 0, 0, 0, 0, 0, 0, 0};
	public final static String []names = {"", "Beamer command", "Authors", "Biblio", "Abstract", "Title", "Figure", "Figure", "", "", "", "", "", "", "", "",
	"Paper", "Article", "Misc", "Tech. Report", "Tech. Report", "Ph.D. Thesis", "Book", "Book", "Manual"};
	public final static String []namesToolTip = {"", "Beamer command", "Authors", "Biblio", "Abstract", "Title", "Figure", "Figure", "Slide", "Slide", "Chapter", "Chapter with no numbering", "Section", "Subsection", "Subsubsection", "Label",
	"Conference paper", "Journal", "Misc", "Technical report", "Technical report","Ph.D. Thesis", "Book", "Book", "Manual"};
	public final static ImageIcon []icons = {IconManager.imgic108, IconManager.imgic108, IconManager.imgic108, IconManager.imgic108, IconManager.imgic102, IconManager.imgic110, IconManager.imgic138, IconManager.imgic138, IconManager.imgic100, IconManager.imgic100, IconManager.imgic104, IconManager.imgic104, IconManager.imgic104, IconManager.imgic106, IconManager.imgic106, IconManager.imgic180, 
	IconManager.imgic66, IconManager.imgic66, IconManager.imgic66, IconManager.imgic66, IconManager.imgic66, IconManager.imgic66, IconManager.imgic66, IconManager.imgic66, IconManager.imgic66};
	private final static boolean []showCurrent = {false, false, false, false, true, false, false, false, true, true, true, true, true, true, true, false,
	false, false, false, false, false, false, false, false, false};
	private final static boolean []isLabel = {false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true,
	false, false, false, false, false, false, false, false, false};
	private final static boolean []isOther = {true, true, true, true, true, true, true, true, false, false, false, false, false, false, false, false,
	true, true, true, true, true, true, true, true, true};
	private final static boolean []isSlide = {false, false, false, false, false, false, false, false, true, true, false, false, false, false, false, false,
	false, false, false, false, false, false, false, false, false};
	private final static boolean []isSection = {false, false, false, false, false, false, false, false, false, false, true, true, true, true, true, false,
	false, false, false, false, false, false, false, false, false};
	private final static boolean []isBibRef = {false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false,
	true, true, true, true, true, true, true, true, true};
	
	
	//public final static String [] names = {"{", "{", "{", "{", "{"}
	
	private int type = 0;
	private String name;
	private LinkedList<LatexElement> elements;
	private int index; // Index in the latex document
	private LatexElement father;
	
	public LatexElement(int _type, String _name, int _index, LatexElement _father) {
		type = _type;
		name = _name;
		index = _index;
		father = _father;
		elements = new LinkedList<LatexElement>();
		//TraceManager.addDev("New element: " + toString());
	}
	
	public boolean isSection() {
		return isSection[getType()];
	}
	
	public boolean isSlide() {
		return isSlide[getType()];
	}
	
	public boolean isOther() {
		return isOther[getType()];
	}
	
	public boolean isLabel() {
		return isLabel[getType()];
	}
	
	public LatexElement getFather() {
		return father;
	}
	
	public void setIndex(int _index) {
		index = _index;
	}
	
	public int getType() {
		return type;
	}
	
	public int getLevel() {
		return levels[type];
	}
	
	public int getIndex() {
		return index;
	}
	
	public void setName(String _name) {
		name = _name;
	}
	
	public String getName() {
		return name;
	}
	
	public void addElement(LatexElement _le) {
		elements.add(_le);
	}
	
	public boolean equals(LatexElement _le) {
		//TraceManager.addDev("Comparing names of " + name + " with " + _le.getName());
		/*if (name.compareTo(_le.getName()) != 0) {
			return false;
		}*/
		
		if (type != _le.getType()) {
			return false;
		}
		
		//TraceManager.addDev("child cound");
		if (_le.getChildCount() != elements.size()) {
			return false;
		}
		//TraceManager.addDev("sub components");
		for (int i=0; i<elements.size(); i++) {
			if (!(elements.get(i).equals((LatexElement)(_le.getChild(i))))) {
				//TraceManager.addDev("Failed on child " + i);
				return false;
			}
		}
		
		return true;
	}
	
	public void updateIndexes(LatexElement _le) {
		if (_le == null) {
			return;
		}
		setIndex(_le.getIndex());
		setName(_le.getName());
		
		for (int i=0; i<elements.size(); i++) {
			elements.get(i).updateIndexes((LatexElement)(_le.getChild(i)));
		}
	}
	
	// path starts at index 1
	public boolean getPath(Object _destination[], Object[] _path, int _startIndex) {
		LatexElement le = (LatexElement)(_path[_startIndex]);
		
		TraceManager.addDev("name comparison in " + le.getName() + " / " + le.getName());
		
		if (le.getName().compareTo(name) != 0) {
			return false;
		}
		
		TraceManager.addDev("type comparison");
		
		if (le.getType() != getType()) {
			return false;
		}
		
		_destination[_startIndex] = this;
		
		
		
		if ((_startIndex+1) == _path.length) {
			return true;
		}
		
		TraceManager.addDev("Next of the path");
		
		for (LatexElement nexts: elements) {
			if (nexts.getPath(_destination, _path, _startIndex+1)) {
				return true;
			}
		}
		
		return false;
		
	}
	
	public String toString() {
		if (getTypeName(type).length() == 0) {
			return name;
		}
		
		//return getTypeName(type) + ": " + name + " (" + index + ")";
		return getTypeName(type) + ": " + name;
	}
	
	public static String getTypeName(int _type) {
		return names[_type];
	}
	
		public int getChildCount() {
		int cpt = 0;
		for(LatexElement le: elements) {
			if (le.isOther() && LatexStructure.showOthers) {
				cpt ++;
			} else if (le.isLabel() && LatexStructure.showLabels){
				cpt ++;
			} else if (le.isSlide() && LatexStructure.showSlides){
				cpt ++;
			}  else if (le.isSection() && LatexStructure.showSections){
				cpt ++;
			}
		}
		
		return cpt;
	}
	
    public Object getChild(int _index) {
		int cpt = 0;
		for(LatexElement le: elements) {
			if (le.isOther() && LatexStructure.showOthers) {
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			} else if (le.isLabel() && LatexStructure.showLabels){
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			}  else if (le.isSlide() && LatexStructure.showSlides){
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			}  else if (le.isSection() && LatexStructure.showSections){
				if (cpt == _index) {
					return le;
				}
				cpt ++;
			}
		}
		
		return null;
	}
	
	public int getIndexOfChild(Object _child) {
		int cpt =0;
		for(LatexElement le: elements) {
			if (le.isOther() && LatexStructure.showOthers) {
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			} else if (le.isSlide() && LatexStructure.showSlides){
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			}  else if (le.isLabel() && LatexStructure.showLabels){
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			}  else if (le.isSection() && LatexStructure.showSections){
				if (le == _child) {
					return cpt;
				}
				cpt ++;
			}
		}
		
		return -1;
	}
	
	public static String getName(String _input, int _index, int _type) {
		//int size = keywords[_type].length();
		int size = 1;
		String s = _input.substring(_index+size, _input.length());
		
		int index= s.indexOf("{");
		
		if (index == -1) {
			return "???";
		}
		
		s = s.substring(index, s.length());
		
		if (isABibRef(_type)) {
			index = s.indexOf(',');
		} else {
			index = LatexStructure.findClosingBracket(s);
		}
		if (index == -1) {
			return "???";
		}
		
		s = s.substring(1, index);
		
		// Frame
		//TraceManager.addDev("Dealing with a frame");
		if (isASlide(_type)){
			//Locate the frametitle directive
			index = s.indexOf("\\frametitle{");
			if (index != -1) {
				String frame = s.substring(index+11, s.length());
				//TraceManager.addDev("frame="+frame);
				index = LatexStructure.findClosingBracket(frame);
				if (index != -1) {
					s = frame.substring(1, index);
				}
			} else {
				index = s.indexOf("\\titlepage");
				if (index != -1) {
					s = "Main slide";
				}
			}
		}
		
		return s;
		
	}
	
	public String getTreeToolTip() {
		return namesToolTip[type];
	}
	
	public static boolean isASlide(int _type) {
		return namesToolTip[_type].compareTo("Slide") == 0;
	}
	
	public static boolean isABibRef(int _type) {
		return isBibRef[_type];
	}
	
	
	
	public ImageIcon getIcon() {
		if (isASlide(type)){
			if (getName().compareTo(MAIN_SLIDE) == 0) {
				return MAIN_SLIDE_ICON;
			}
		}
		return icons[type];
	}
	
	public LatexElement getVeryLastLatexElement() {
		if (elements.size() == 0) {
			return this;
		}
		
		return elements.get(elements.size()-1).getVeryLastLatexElement();
	}
	
	public LatexElement getJustBeforeIndex(int _index) {
		LatexElement foundLe, le;
		for(int i=elements.size()-1; i>-1; i--) {
			le = elements.get(i);
			foundLe = le.getJustBeforeIndex(_index);
			if (foundLe != null) {
				return foundLe;
			}
		}
		
		if ((getIndex() < _index) && (showCurrent[getType()])) {
			return this;
		}
		
		return null;
		
	}
	
	public LatexElement getAfterIndex(int _index) {
		if ((getIndex() > _index) && (showCurrent[getType()])) {
			return this;
		}
		
		LatexElement foundLe;
		for(LatexElement le: elements) {
			foundLe = le.getAfterIndex(_index);
			if (foundLe != null) {
				return foundLe;
			}
		}
		
		return null;
		
	}
	
} // Class 

