/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille

ludovic.apvrille AT enst.fr

This software is a computer program whose purpose is to allow the
edition of Latex documents

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

/**
* Class GBHelper
* Creation: See below
* Version 2.0 04/01/2012
* @author Ludovic APVRILLE
* @see
*/


// File   : layoutDemos/layoutDemoGB/GBHelper.java
// Purpose: Keeps track of current position in GridBagLayout.
//          Supports a few GridBag features: position, width, height, expansion.
//          All methods return GBHelper object for call chaining.
// Author : Fred Swartz - January 30, 2007 - Placed in public domain.


package myutil;

import java.awt.*;

//////////////////////////////////////////////////////////////////// Class
public class GBHelper extends GridBagConstraints {
    
    //============================================================== constructor
    /* Creates helper at top left, component always fills cells. */
    public GBHelper() {
        gridx = 0;
        gridy = 0;
        fill = GridBagConstraints.BOTH;  // Component fills area 
    }
    
    //================================================================== nextCol
    /* Moves the helper's cursor to the right one column. */
    public GBHelper nextCol() {
        gridx++;
        return this;
    }
    
    //================================================================== nextCol
    /* Moves the helper's cursor to the right "col" column. */
    public GBHelper nextCol(int col) {
        gridx += col;
        return this;
    }
    
    //================================================================== nextRow
    /* Moves the helper's cursor to first col in next row. */
    public GBHelper nextRow() {
        gridx = 0;
        gridy++;
        return this;
    }
    
    //================================================================== expandW
    /* Expandable Width.  Returns new helper allowing horizontal expansion. 
       A new helper is created so the expansion values don't
       pollute the origin helper. */
    public GBHelper expandW() {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.weightx = 1.0;
        return duplicate;
    }
    
    //================================================================== expandH
    /* Expandable Height. Returns new helper allowing vertical expansion. */
    public GBHelper expandH() {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.weighty = 1.0;
        return duplicate;
    }
    
    //==================================================================== width
    /* Sets the width of the area in terms of number of columns. */
    public GBHelper width(int colsWide) {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.gridwidth = colsWide;
        return duplicate;
    }
    
    //==================================================================== width
    /* Width is set to all remaining columns of the grid. */
    public GBHelper width() {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.gridwidth = REMAINDER;
        return duplicate;
    }
    
    //=================================================================== height
    /* Sets the height of the area in terms of rows. */
    public GBHelper height(int rowsHigh) {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.gridheight = rowsHigh;
        return duplicate;
    }
    
    //=================================================================== height
    /* Height is set to all remaining rows. */
    public GBHelper height() {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.gridheight = REMAINDER;
        return duplicate;
    }
    
    //==================================================================== align
    /* Alignment is set by parameter. */
    public GBHelper align(int alignment) {
        GBHelper duplicate = (GBHelper)this.clone();
        duplicate.fill   = NONE;
        duplicate.anchor = alignment;
        return duplicate;
    }
}
