/**Copyright or (C) or Copr. GET / ENST, Telecom-Paris, Ludovic Apvrille
*
* ludovic.apvrille AT enst.fr
*
* This software is a computer program whose purpose is to allow the
* edition of Latex documents
*
* This software is governed by the CeCILL  license under French law and
* abiding by the rules of distribution of free software.  You can  use,
* modify and/ or redistribute the software under getthe terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info".
*Sc
* As a counterpart to the access to the source code and  rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability.
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or
* data to be ensured and,  more generally, to use and operate it in the
* same conditions as regards security.
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL license and that you accept its terms.
*
* /**
* Class MainGUI
* Main window / Actions management / Mode management
* Creation: 11/06/2010
* @version 1.0 11/06/2010
* @author Ludovic APVRILLE
* @see
*/

package ui;

import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import java.awt.image.*;
import javax.imageio.*;

import java.nio.charset.*;
import java.nio.*;

import latexstructure.*;
import myutil.*;
import template.*;
import ui.file.*;
import ui.latexedition.*;
import ui.window.*;

import javax.activation.*;




public	class MainGUI implements ActionListener, WindowListener, KeyListener {
    
    public JFrame frame; //Main Frame
    public Container framePanel; //Main pane
    public JSplitPane split, split1;
	public JPanelInfo panelForInfo;
	//public JPanelForEdition panelForEdition;
	public JPanelForCompilation panelForCompilation;
	
	//protected JFrameBasicText latexFrame;
    
    public int indexPrevious = 0;
    
    // JTree
	//public SlidePanel slidePanel
    
    // Actions
    public	LEGUIAction [] actions;
    public	MouseHandler mouseHandler;
    public  KeyListener keyHandler;
    
    
    // Status bar
    private	JLabel status;
    
    //Menubar
    private JMenuBarLatexEditor jmenubar;
    //public final static boolean analysis = false;
    
	protected int mode;
    public final static byte NOT_OPENED = 0;
    public final static byte OPENED = 1;
	public final static byte SAVED = 2;
	public final static byte CURRENT_SAVED = 3;
	public final static byte COMPILING = 4;
	public final static byte COMPILING_DONE = 5;
	public final static byte CURRENT_TO_SAVE = 6;  
	public final static byte GENERATE_CURRENT = 7;
	public final static byte NOT_GENERATE_CURRENT = 8;
	public final static byte NOT_FIND = 9;
	public final static byte FIND = 10;
	
	
	public final static byte UNDO_SIZE = 25;
    
    //public final static int INCREMENT = 10;
	
    
    // Latex Structure
    private EditionSet currentEdition;
	private LinkedList<EditionSet> editions;
	
	// Templates
	LinkedList<Template> templates;
	
	// Dictionary
	private ArrayList<MyDictionary> dictionaries;
    
    //TURTLE modeling graphic components
    private JTabbedPane mainTabbedPane;
	private JToolBarMainLatexEditor mainBar;
    //private JPopupMenu menuTabbedPane;
	private ScrolledJTextArea jta;
    
    //private File file;
	
    private JFileChooser jfc;
    private JFrameFind findFrame;
	
	private Autosave autosave;
	
	private String find = null;
	private String replace = "";
	private boolean matchCase = false;
	private String loadMode = null;
    
    
    public MainGUI() {
		editions = new LinkedList<EditionSet>();
		dictionaries = new ArrayList<MyDictionary>();
    }
	
	public void setTemplates(LinkedList<Template> _templates) {
		templates = _templates;
	}
	
	public LinkedList<Template>  getTemplates() {
		return templates;
	}
    
    
    public void build() {
        
        // Swing look and feel
        
        try {
            UIManager.setLookAndFeel(
				UIManager.getCrossPlatformLookAndFeelClassName());
        } catch	(Exception e) {	ErrorGUI.exit(ErrorGUI.GUI);}
        
        // Creating main container
        frame =	new JFrame("LOBSTER - Latex and Beamer Editor");
        frame.addWindowListener(this);
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE );
        //frame.setIconImage(IconManager.imgic10);
        
        framePanel = frame.getContentPane();
        framePanel.setLayout(new BorderLayout());
        
        // file chooser
        if (ConfigurationLatexEditor.DocumentPath.length() > 0) {
            jfc = new JFileChooser(ConfigurationLatexEditor.DocumentPath);
        } else {
            jfc = new JFileChooser();
        }
        
        LatexFileFilter filter = new LatexFileFilter();
        jfc.setFileFilter(filter);
        
        // Actions
        initActions();
        
        // mode
        setMode(NOT_OPENED);
        
        // statusBar
        status = createStatusBar();
        
        // Mouse handler
        mouseHandler = new MouseHandler(status);
        
        framePanel.add(status, BorderLayout.SOUTH);
        
        // toolbar
		mainBar = new JToolBarMainLatexEditor(this);
        framePanel.add(mainBar, BorderLayout.NORTH);
        
        // Panels
		mainTabbedPane = new JTabbedPane();
		ChangeListener cl = new	ChangeListener() {
            public void stateChanged(ChangeEvent e){
                paneAction(e);
            }
        };
        mainTabbedPane.addChangeListener(cl);
        //panelForEdition = new JPanelForEdition(this);
		//panelForCompilation = new JPanelForCompilation(this);
		//panelForCompilation.setSize(300, 150);
		//split1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, mainTabbedPane, panelForCompilation);
		//split1.setResizeWeight(1.0);
		//split1.setDividerLocation(0.90);
		//panelForCompilation.setPreferredSize(new Dimension(200, 100));
		
		panelForInfo = new JPanelInfo(this);
		panelForCompilation = panelForInfo.getJPanelForCompilation();
		Dimension minimumSize = new Dimension(0, 0);
		panelForInfo.setMinimumSize(minimumSize);
		Dimension preferredDim = new Dimension(600, 400);
        mainTabbedPane.setPreferredSize(preferredDim);
        preferredDim = new Dimension(100, 400);
        panelForInfo.setPreferredSize(preferredDim);
        split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, panelForInfo, mainTabbedPane);
        framePanel.add(split, BorderLayout.CENTER);
        
        //split.setResizeWeight(0.8);
        //split.setDividerLocation(0.20);
        // Creating menus
        jmenubar = new JMenuBarLatexEditor(this);
        frame.setJMenuBar(jmenubar);
		
		autosave = new Autosave(this);
        
    }
	
	public void setTitle(String title) {
		frame.setTitle("Lobster: " + title);
	}
    
    private	void initActions() {
        actions = new LEGUIAction[LEGUIAction.NB_ACTION];
        for(int	i=0; i<LEGUIAction.NB_ACTION; i++) {
            actions[i] = new LEGUIAction(i);
            actions[i].addActionListener(this);
            //actions[i].addKeyListener(this);
        }
    }
	
	public void addDictionary(MyDictionary _dictionary) {
		dictionaries.add(_dictionary);
	}
	
	public MyDictionary getDictionary(int _index) {
		return dictionaries.get(_index);
	}
	
	public HashSet getDictionaryHash(int _index) {
		return dictionaries.get(_index).getDictionaryHash();
	}
	
	public boolean hasDictionaries() {
		return dictionaries.size() > 0;
	}
	
	public int getNbOfDictionaries() {
		return dictionaries.size();
	}
	
	public void updateOnDictionaries() {
		if (currentEdition != null) {
			currentEdition.updateText();
		}
	}
	
	public void setLoadMode(String _mode) {
		loadMode = _mode;
	}
	
	public void manageFind(String _text) {
		//TraceManager.addDev("Manage find = " + _text);
		find = _text;
		if (currentEdition != null) {
			//TraceManager.addDev("No null edition");
			currentEdition.manageFind(_text);
			if ((_text == null) || (_text.trim().length() == 0)) {
				setFindMode(NOT_FIND);
			} else {
				setFindMode(FIND);
			}
		}
	}
	
	public void manageReplace(String _text) {
		replace = _text;
	}
	
	public void replaceOne() {
		if (currentEdition != null) {
			currentEdition.replaceOne(find, replace);
		}
	}
	
	public void replaceAll() {
		if (currentEdition != null) {
			currentEdition.replaceAll(find, replace);
		}
	}
	
	public void setMatchCase(boolean _b) {
		matchCase = _b;
		if (currentEdition != null) {
			currentEdition.setMatchCase(_b);
		}
	}
	
	public void applyFind() {
		if (currentEdition != null) {
			currentEdition.updateFind();
		}
	}
	
	public void findPrevious() {
		if (currentEdition != null) {
			currentEdition.findPrevious();
		}
	}
	
	public void findFrame() {
		if (JFrameFind.opened == 0){
			findFrame = new JFrameFind("Find", this);
			//findFrame.setIconImage(IconManager.img8);
			
			findFrame.setSize(450, 250);
			
			GraphicLib.centerOnParent(findFrame);
			
			findFrame.setVisible(true);
        }
	}
	
	// If upperCase is set to true, the same word with iots first letter as uppercase
	// is also added to dictionary
	public void addToPersonalDictionary(String _word) {
		//System.out.println("Adding to personal dic:" + _word);
		
		try {
			String fileName = ConfigurationLatexEditor.PersonalDictionary;
			if (fileName.charAt(0) == '*') {
				fileName = fileName.substring(1, fileName.length());
			}
			fileName = fileName.trim();
			
			setStatusBarText("Adding word \"" + _word + "\" to " + ConfigurationLatexEditor.DataPath + fileName);
			
			String data = FileUtils.loadFile(ConfigurationLatexEditor.DataPath + fileName);
			data = data + "\n" + _word;
			FileUtils.saveFile(ConfigurationLatexEditor.DataPath + fileName, data);
			
			dictionaries.get(dictionaries.size()-1).addToDictionary(_word);
			if (Character.isLowerCase(_word.charAt(0))) {
				dictionaries.get(dictionaries.size()-1).addToDictionary(_word.charAt(0) + _word.substring(1, _word.length()));
			}
			dictionaries.get(dictionaries.size()-1).addToDictionary(_word);
			
			if (currentEdition != null) {
				currentEdition.strongUpdateText();
			}
			
			setStatusBarText("Word \"" + _word + "\" added in " + ConfigurationLatexEditor.DataPath + fileName);
		} catch (Exception e) {
			System.err.println("Adding \"" + _word + "\" to personal dictionary failed:" + e.getMessage());
			JOptionPane.showMessageDialog(frame, "Adding \"" + _word + "\" to personal dictionary failed:" + e.getMessage(), "Error", JOptionPane.INFORMATION_MESSAGE);
			return;
		}
	}
	
	private void updateCurrentText() {
		if (currentEdition != null) {
			currentEdition.updateText();
		}
	}
    
    public String getTitle() {
        return frame.getTitle();
    }
	
	public byte getMode() {
		return (byte)mode;
	}
	
	public void setUndoRedo(int _indexLS, int _size) {
		actions[LEGUIAction.ACT_BACKWARD].setEnabled((_size>1) && (_indexLS > 0));
		actions[LEGUIAction.ACT_FORWARD].setEnabled((_size>1) && (_indexLS < (_size-1)));
	}
    
	
	public void setFindMode(byte m) {
		// not all mode
		switch(m) {
		case NOT_FIND:
			actions[LEGUIAction.FIND].setEnabled(false);
			actions[LEGUIAction.FIND_PREVIOUS].setEnabled(false);
			actions[LEGUIAction.REPLACE_ONE].setEnabled(false);
			actions[LEGUIAction.REPLACE_ALL].setEnabled(false);
			return;
		case FIND:
			actions[LEGUIAction.FIND].setEnabled(true);
			actions[LEGUIAction.FIND_PREVIOUS].setEnabled(true);
			actions[LEGUIAction.REPLACE_ONE].setEnabled(true);
			actions[LEGUIAction.REPLACE_ALL].setEnabled(true);
			return;
		default:
			TraceManager.addDev("DEFAULT");
		}
	}
	
	
    public void setMode(byte m) {
    	
    	
        mode = m;
		actions[LEGUIAction.ACT_SAVE_ALL].setEnabled(hasOneToSave());
		if (currentEdition == null) {
			actions[LEGUIAction.ACT_SAVE].setEnabled(false);
		} else {
			actions[LEGUIAction.ACT_SAVE].setEnabled(!currentEdition.isSaved());
		}
        switch(mode) {
		case MainGUI.NOT_OPENED:
			activeActions(false);
			actions[LEGUIAction.ACT_NEW].setEnabled(true);
			actions[LEGUIAction.ACT_OPEN].setEnabled(true);
			actions[LEGUIAction.ACT_LAST_OPEN].setEnabled(ConfigurationLatexEditor.LastOpenFileDefined);
			actions[LEGUIAction.ACT_QUIT].setEnabled(true);
			actions[LEGUIAction.ABOUT].setEnabled(true);
			break;
		case OPENED:
			actions[LEGUIAction.FIND_FRAME].setEnabled(false);
			actions[LEGUIAction.ACT_CLOSE].setEnabled(true);
			actions[LEGUIAction.GENERATE_NOTES].setEnabled(true);
			//actions[LEGUIAction.ACT_SAVE_ALL].setEnabled(true);
			actions[LEGUIAction.ACT_SAVE_AS].setEnabled(true);
			//actions[LEGUIAction.ACT_BACKWARD].setEnabled((_size>1) && (_indexLS > 0));
			//actions[LEGUIAction.ACT_FORWARD].setEnabled((_size>1) && (_indexLS < (_size-1)));
			actions[LEGUIAction.ACT_INDENT].setEnabled(true);
			actions[LEGUIAction.ACT_COMPILE_ONLY].setEnabled(true);
			//actions[LEGUIAction.ACT_COMPILE].setEnabled(true);
			actions[LEGUIAction.ACT_COMPILE_ALL].setEnabled(true);
			actions[LEGUIAction.ACT_MAKE_CLEAN].setEnabled(true);
			actions[LEGUIAction.ACT_MAKE_ULTRACLEAN].setEnabled(true);
			actions[LEGUIAction.ACT_VIEW_PDF].setEnabled(true);
			actions[LEGUIAction.ACT_VIEW_PDF_CURRENT].setEnabled(true);
			actions[LEGUIAction.ACT_VIEW_PDF_NOTES].setEnabled(true);
			break;     
		case SAVED:
			//actions[LEGUIAction.ACT_SAVE].setEnabled(false);
			actions[LEGUIAction.ACT_CLOSE].setEnabled(true);
			actions[LEGUIAction.GENERATE_NOTES].setEnabled(true);
			break;    
		case COMPILING:
			actions[LEGUIAction.ACT_COMPILE_ONLY].setEnabled(false);
			actions[LEGUIAction.ACT_COMPILE].setEnabled(false);
			actions[LEGUIAction.ACT_COMPILE_ALL].setEnabled(false);
			actions[LEGUIAction.ACT_MAKE_CLEAN].setEnabled(false);
			actions[LEGUIAction.ACT_MAKE_ULTRACLEAN].setEnabled(false);
			break;
		case COMPILING_DONE:
			actions[LEGUIAction.ACT_COMPILE_ONLY].setEnabled(true);
			if (actions[LEGUIAction.GENERATE_CURRENT].isEnabled()) {
				actions[LEGUIAction.ACT_COMPILE].setEnabled(true);
			}
			actions[LEGUIAction.ACT_COMPILE_ALL].setEnabled(true);
			actions[LEGUIAction.ACT_MAKE_CLEAN].setEnabled(true);
			actions[LEGUIAction.ACT_MAKE_ULTRACLEAN].setEnabled(true);
			break;
		case GENERATE_CURRENT:
			actions[LEGUIAction.GENERATE_CURRENT].setEnabled(true);
			actions[LEGUIAction.ACT_COMPILE].setEnabled(true);
			break;
		case NOT_GENERATE_CURRENT:
			actions[LEGUIAction.GENERATE_CURRENT].setEnabled(false);
			actions[LEGUIAction.ACT_COMPILE].setEnabled(false);
			break;
		default:
			TraceManager.addDev("DEFAULT");
			activeActions(false);
        }
    }
	
    
    
    private void activeActions(boolean b) {
        for(int	i=0; i<LEGUIAction.NB_ACTION; i++) {
            actions[i].setEnabled(b);
        }
    }
    
    
    // Creates the status bar.
    private	JLabel createStatusBar()  {
        status = new JLabel("Ready...");
        status.setBorder(BorderFactory.createEtchedBorder());
        return status;
    }
    
    public void activate(JMenu jm, boolean b) {
        JMenuItem im;
        for(int	i=0; i<jm.getItemCount(); i++) {
            im = jm.getItem(i);
            if (im != null)
                im.setEnabled(b);
        }
    }
    
    public void start() {
        // Main	window is ready	to be drawn on screen
        if (frame == null) {
            ErrorGUI.exit(ErrorGUI.GUI);
        }
		
		boolean positioned = false;
		
		if (ConfigurationLatexEditor.configSizeAvailable()) {
			try {
				int x = Integer.decode(ConfigurationLatexEditor.LastWindowAttributesX).intValue();
				int y = Integer.decode(ConfigurationLatexEditor.LastWindowAttributesY).intValue();
				int width = Integer.decode(ConfigurationLatexEditor.LastWindowAttributesWidth).intValue();
				int height = Integer.decode(ConfigurationLatexEditor.LastWindowAttributesHeight).intValue();
				String max = ConfigurationLatexEditor.LastWindowAttributesMax;
				if (max.compareTo("true") != 0) {
					frame.setBounds(x, y, width, height);
					positioned = true;
				}
				//System.out.println("Setting window attributes");
			} catch (Exception e) {
				//System.out.println("Window positioning has failed: " + e.getMessage()); 
			}
		}
		
		if (!positioned) {
			frame.setBounds(100, 100, 800, 600);
			// jdk 1.4 or more
        	frame.setExtendedState(frame.getExtendedState() | JFrame.MAXIMIZED_BOTH);
			//System.out.println("No default window attributes");
		}
        
        frame.setVisible(true);
		//split1.setDividerLocation(0.80);
    }
	
	public LatexStructure getLatexStructure() {
		if (currentEdition != null) {
			return currentEdition.getLatexStructure();
		}
		return null;
	}
	
	/*public File getCurrentFile() {
	return file;
	}*/
	
	public LinkedList<EditionSet> getEditionSet() {
		return editions;
	}
	
	public void updatePanels() {
		if (currentEdition != null) {
			currentEdition.updatePanels();
			panelForInfo.setNewStructure(currentEdition.getLatexStructure());
			panelForInfo.changeMade();
		} else {
			panelForInfo.setNewStructure(null);
			panelForInfo.applyChange();
		}
		
		
	}
	
	public void updateInfoPanel() {
		panelForInfo.setNewStructure(currentEdition.getLatexStructure());
		panelForInfo.changeMade();
		manageFind(find);
	}
	
	public void selectedElementDocument(int _index) {
		if (currentEdition != null) {
			currentEdition.selectedElementDocument(_index);
		}
	}
    
    /*public void closeLatexDocument() {
    if (mode != NOT_OPENED) {
    setMode(NOT_OPENED);
    frame.setTitle("Latex Editor");
    frame.repaint();
    }
    }*/
    
    public void setStatusBarText(String s) {
        // captitalizeFirstLetter
        status.setText(s.substring(0, 1).toUpperCase() + s.substring(1, s.length()));
    }
    
    
    // Action listeners
    public void newProject() {
		EditionSet es = new EditionSet(this);
		editions.add(es);
		es.setMatchCase(matchCase);
		es.manageFind(find);
		mainTabbedPane.add("unamed", es.getJPanelForEdition());
		mainTabbedPane.setTabComponentAt(editions.size()-1, new ButtonTabComponent(this, mainTabbedPane));
		mainTabbedPane.setSelectedIndex(editions.size()-1);
		openDone();
    }
    
    public String loadFile(File f) {
        String s = null;
        
        if(checkFileForOpen(f)) {
            try {
                FileInputStream fis = new FileInputStream(f);
                int nb = fis.available();
                
                byte [] ba = new byte[nb];
                fis.read(ba);
                fis.close();
                s = removeantishlashr(new String(ba));
            } catch(Exception e) {
                JOptionPane.showMessageDialog(frame, "File could not be opened because " + e.getMessage(), "File Error", JOptionPane.INFORMATION_MESSAGE);
            }
        }
        return s;
    }
    
    public void saveFile(File f, String data, String msg) {
        if(checkFileForSave(f)) {
            try {
                if (data == null) {
                    return;
                }
                FileOutputStream fos = new FileOutputStream(f);
                fos.write(data.getBytes());
                fos.close();
                JOptionPane.showMessageDialog(frame, msg + " " + f.getAbsolutePath(), "Saving", JOptionPane.INFORMATION_MESSAGE);
                return;
            } catch(Exception e) {
                JOptionPane.showMessageDialog(frame, "File could not be saved because " + e.getMessage(), "File Error", JOptionPane.INFORMATION_MESSAGE);
                return;
            }
        }
    }
    
    public void updateLastOpenFile() {
        if (ConfigurationLatexEditor.LastOpenFileDefined) {
			String s = "";
			String s1 = "";
			int cpt = 0;
			for(EditionSet es: editions) {
				File file = es.getFile();
				if (file != null) {
					if (cpt > 0) {
						s += " ; ";
						s1 += " ; ";
					}
					s += file.getPath();
					s1 += file.getName();
					cpt ++;
				}
			}
            ConfigurationLatexEditor.LastOpenFile = s;
            // Change name of action
            
            actions[LEGUIAction.ACT_LAST_OPEN].setName(LEGUIAction.ACT_LAST_OPEN, "Open last documents: " + s1);
            actions[LEGUIAction.ACT_LAST_OPEN].setShortDescription(LEGUIAction.ACT_LAST_OPEN, "Open files: " + s1);
			actions[LEGUIAction.ACT_LAST_OPEN].setLongDescription(LEGUIAction.ACT_LAST_OPEN, "Open the lastly saved Latex Presentations: " + s1);
        }
    }
    
    
    public void openProject() {
		File file = null;
		int returnVal = jfc.showOpenDialog(frame);
        
		if (returnVal == JFileChooser.CANCEL_OPTION) {
			return;
		}
		
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            file = jfc.getSelectedFile();
        }
        
        String s = null;
        String encoding = null;
		
		if(checkFileForOpen(file)) {
            try {
            	//System.out.println("Mime Type of " + file.getName() + " is " +
            	//        new MimetypesFileTypeMap().getContentType(file));
            	
                FileInputStream fis = new FileInputStream(file);
                int nb = fis.available();
                
                byte [] ba = new byte[nb];
                fis.read(ba);
                fis.close();
                
                if (loadMode == null) {
                	encoding = FileUtils.getEncoding(file.getAbsolutePath());
                } else {
                	encoding = loadMode.toUpperCase();
                }
                TraceManager.addDev("File. using external function; encoding = " + encoding);
                s = removeantishlashr(new String(ba, encoding));
                
            } catch(Exception e) {
                JOptionPane.showMessageDialog(frame, "File could not be opened because " + e.getMessage(), "File Error", JOptionPane.INFORMATION_MESSAGE);
                return;
            }
			//TraceManager.addDev("New edition set");
			EditionSet es = new EditionSet(this, s);
			editions.add(es);
			es.setFile(file);
			es.setMatchCase(matchCase);
			es.setEncoding(encoding);
			es.manageFind(find);
			mainTabbedPane.add(file.getName(), es.getJPanelForEdition());
			mainTabbedPane.setTabComponentAt(editions.size()-1, new ButtonTabComponent(this, mainTabbedPane));
			mainTabbedPane.setSelectedIndex(editions.size()-1);
			es.selectedElementDocument(0);
			openDone();
			//TraceManager.addDev("Done done");
			es.setSaved(true);
		}
	}
	
	public void openDone() {
		setMode(OPENED);
		MakeChangeOnInfoPanel();
	}
	
	public void MakeChangeOnInfoPanel() {
		panelForInfo.changeMade();
		panelForInfo.applyChange();
	}
    
    public void openLastProject() {
        String sf = ConfigurationLatexEditor.LastOpenFile;
		
		String [] ss = sf.split(";");
		String encoding;
		
		for(int i=0; i<ss.length; i++) {
			File file = new File(ss[i].trim());
			
			String s = null;
			if(checkFileForOpen(file)) {
				try {
					FileInputStream fis = new FileInputStream(file);
					int nb = fis.available();
					
					byte [] ba = new byte[nb];
					fis.read(ba);
					fis.close();
					if (loadMode == null) {
					encoding = FileUtils.getEncoding(file.getAbsolutePath());
					} else {
						encoding = loadMode.toUpperCase();
					}
					TraceManager.addDev("File. using external function; encoding = " + encoding);
					s = removeantishlashr(new String(ba, encoding));
					//s = removeantishlashr(new String(ba));
				} catch(Exception e) {
					JOptionPane.showMessageDialog(frame, "File could not be opened because " + e.getMessage(), "File Error", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
				EditionSet es = new EditionSet(this, s);
				editions.add(es);
				es.setFile(file);
				es.setMatchCase(matchCase);
				es.setEncoding(encoding);
				es.manageFind(find);
				mainTabbedPane.add(file.getName(), es.getJPanelForEdition());
				mainTabbedPane.setTabComponentAt(editions.size()-1, new ButtonTabComponent(this, mainTabbedPane));
				mainTabbedPane.setSelectedIndex(editions.size()-1);
				es.selectedElementDocument(0);
				openDone();
				es.setSaved(true);
			}
		}
    }
    
    public void saveAsProject() {
		if (currentEdition == null) {
			return;
		}
		File file = currentEdition.getFile();
        File old = file;
        if (file == null) {
            // project never saved
            saveProject();
        } else {
            currentEdition.setFile(null);
            if (!saveProject()) {
                file = old;
				currentEdition.setFile(old);
            }
        }
		file = currentEdition.getFile();
        if ((file != old) && (file != null)) {
            frame.setTitle(file.getAbsolutePath());
			if (currentEdition.getLatexStructure() != null) {
				currentEdition.getLatexStructure().setName(file.getName());
				//panelForInfo.updateList();
			}
			currentEdition.applyMode();
        }
    }
    
    public boolean saveProject() {
		if (currentEdition == null) {
			return false;
		}
		
		return saveProject(currentEdition);
	}
	
	public boolean closeCurrentProject() {
		if (currentEdition == null) {
			return false;
		}
		
		return closeProject(editions.indexOf(currentEdition));
	}
	
	public boolean closeProject(int _index) {
		EditionSet es = editions.get(_index);
		if (es == null) {
			return false;
		}
		
		if (!es.isSaved()) {
			// Ask for save
			Object[] options = {"Save", "Discard changes", "CANCEL"};
			int back = JOptionPane.showOptionDialog(frame, "Document has not been saved", "Warning",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);
			//System.out.println("back= " + back);
			if (back == JOptionPane.CANCEL_OPTION) {
				return false;
			}
			if (back == JOptionPane.YES_OPTION) {
				if (!saveProject(es)) {
					TraceManager.addDev("Not saved");
					return false;
				}
				TraceManager.addDev("Saved");
			}
		}
		
		
		if (_index > -1) {
			editions.remove(_index);
			mainTabbedPane.remove(_index);
		}
		
		if (editions.size() == 0) {
			currentEdition = null;
			setMode(NOT_OPENED);
			TraceManager.addDev("Setting to null");
			panelForInfo.setNewStructure(null);
			panelForInfo.repaint();
			TraceManager.addDev("Setting to null done");
		}
		
		return true;
	}
	
	public boolean hasOneToSave() {
		boolean b;
		for(EditionSet es: editions) {
			if (!es.isSaved()) {
				return true;
			}
		}
		return false;
	}
	
	public boolean saveAllProjects() {
		boolean b;
		for(EditionSet es: editions) {
			if (!es.isSaved()) {
				saveProject(es);
			}
		}
		return true;
	}
	
	public boolean saveProject(EditionSet _es) {
		
		File file = _es.getFile();
		
        if (file == null) {
            //jfc.setApproveButtonText("Save");
            int returnVal = jfc.showSaveDialog(frame);
            if(returnVal == JFileChooser.APPROVE_OPTION) {
                file = jfc.getSelectedFile();
                file = FileUtils.addFileExtensionIfMissing(file, LatexFileFilter.getExtension());
            }
        }
        
        if(checkFileForSave(file)) {
        	int index = _es.getSelectedCharSet();
            try {
                if (_es.getLatexStructure() == null) {
                    throw new Exception("Internal model Error 1");
                }
				TraceManager.addDev("Saving in file:" + file);
				//TraceManager.addDev("Text:" + _es.getLatexStructure().getText());
                FileOutputStream fos = new FileOutputStream(file);
                TraceManager.addDev("Saving in format: " + EditionSet.charenc[index]);
                if (index == 0) {
                	fos.write(_es.getLatexStructure().getText().getBytes("UTF-8"));
                } else {
                	fos.write(_es.getLatexStructure().getText().getBytes(EditionSet.charenc[index]));
                }
                fos.close();
                updateLastOpenFile();
				index = editions.indexOf(_es);
				if (index > -1) {
					mainTabbedPane.setTitleAt(index, file.getName());
					setTitle(file.getAbsolutePath());
				}
				_es.setSaved(true);
				setMode(OPENED);
				setStatusBarText("Saved in " + file);
                return true;
            } catch(Exception e) {
                JOptionPane.showMessageDialog(frame, "File could not be saved because " + e.getMessage(), "File Error", JOptionPane.INFORMATION_MESSAGE);
                return false;
            }
        }
		return false;
    }
	
	public boolean generateNotes() {
		if (currentEdition == null) {
			return false;
		}
		
		return generateNotes(currentEdition);
	}
	
	public boolean generateCurrent() {
		if (currentEdition == null) {
			return false;
		}
		
		return generateCurrent(currentEdition);
	}
	
	public boolean generateNotes(EditionSet _es) {
		
		File file = _es.getFile();
		
        if (file == null) {
            return false;
        }
		
		String notes = _es.getLatexStructure().getNotes(stringToLatex(file.getName()));
		String name = FileUtils.addBeforeFileExtension(file.getAbsolutePath(), "_notes");
        
        try {
			FileUtils.saveFile(name, notes);
			setStatusBarText("Notes generated in " + name);
		} catch (Exception e) {
			setStatusBarText("Generation of notes failed in " + name + ": " + e.getMessage());
		}
		
		return true;
    }
	
	public boolean generateCurrent(EditionSet _es) {
		
		File file = _es.getFile();
		
        if (file == null) {
            return false;
        }
		
		String current = _es.getLatexStructure().getCurrent(_es.getJPanelForEdition().getFirstIndex(), _es.getJPanelForEdition().getEditionText(), stringToLatex(file.getName()));
		String name = FileUtils.addBeforeFileExtension(file.getAbsolutePath(), "_current");
        
        try {
			FileUtils.saveFile(name, current);
			setStatusBarText("Current generated in " + name);
		} catch (Exception e) {
			setStatusBarText("Generation of current failed in " + name + ": " + e.getMessage());
		}
		
		return true;
    }
    
    public boolean checkFileForOpen(File file) {
        boolean ok = true;
        String pb = "";
        
        //System.out.println("File path=" + file.getPath() + " name=" + file.getName());
        
        if (file == null) {
            return false;
        }
        
        try {
            if (file != null) {
                if (!file.exists()) {
                    pb  = "File doesn't exit";
                    ok = false;
                }
                if ((ok == true) && (!file.canRead())) {
                    pb  = "File is read protected";
                    ok = false;
                }
            }
        } catch (Exception e) {
            ok = false;
            pb = e.getMessage();
        }
        if (ok == false) {
            file = null;
            JOptionPane.showMessageDialog(frame, pb, "File Error", JOptionPane.INFORMATION_MESSAGE);
        }
        return ok;
    }
    
    public boolean checkFileForSave(File file) {
        boolean ok = true;
        String pb = "";
        
        if (file == null) {
            return false;
        }
        
        try {
            if (file != null) {
                if (!file.exists()) {
                    if (!file.createNewFile()) {
                        pb  = "File could not be created";
                        ok = false;
                    }
                    if (!file.canWrite()) {
                        pb  = "File is write protected";
                        ok = false;
                    }
                }
            }
        } catch (Exception e) {
            ok = false;
            pb = e.getMessage();
        }
        if (ok == false) {
            file = null;
            JOptionPane.showMessageDialog(frame, pb, "File Error", JOptionPane.INFORMATION_MESSAGE);
        }
        return ok;
    }
    
    public boolean saveBeforeAction(String str1, String str2, String str3) {
        Object[] options = { str1, str2, "CANCEL" };
        int back = JOptionPane.showOptionDialog(frame, "Documents " + str3 + " have not been saved", "Warning",
			JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
			null, options, options[0]);
        //System.out.println("back= " + back);
        if (back == JOptionPane.CANCEL_OPTION) {
            return false;
        }
        if (back == JOptionPane.YES_OPTION) {
            saveAllProjects();
        }
        return true;
    }
    
    public void quitApplication() {
		String name = "";
		boolean b;
		for(EditionSet es: editions) {
			
			if (!es.isSaved()) {
				File file = es.getFile();
				if (file != null) {
					name += file.getName() +  " ";
				}
			}
		}
		
		if (name.length() > 0) {
			if (saveBeforeAction("SAVE and QUIT", "QUIT NOW", name) == false) {
				return;
			}
			
		}
		
		ConfigurationLatexEditor.LastWindowAttributesX = "" + frame.getLocation().x;
		ConfigurationLatexEditor.LastWindowAttributesY = "" + frame.getLocation().y;
		ConfigurationLatexEditor.LastWindowAttributesWidth = "" + frame.getSize().width;
		ConfigurationLatexEditor.LastWindowAttributesHeight = "" + frame.getSize().height;
		
		if (frame.getExtendedState() == Frame.MAXIMIZED_BOTH) {
			ConfigurationLatexEditor.LastWindowAttributesMax = "true";
		} else {
			ConfigurationLatexEditor.LastWindowAttributesMax = "false";
		}
		
		try {
			if (ConfigurationLatexEditor.LastOpenFileDefined) {
				ConfigurationLatexEditor.saveConfiguration();
			}
			} catch (Exception e) {}
			
			System.exit(0);
    }
    
	
	/*public void setNewSelectedElement(int _lineNb) {
	setMode(CURRENT_TO_SAVE);
	selectedLine = _lineNb;
	panelForEdition.setLatexLine(selectedLine);
	}*/
	
	public void changeFromEdition() {
		currentEdition.changeOnLS();
		setMode(OPENED);
		/*setMode(OPENED);
		addPreviousLS(ls);*/
		panelForInfo.changeMade();
	}
	
	//public void changeFromLS() {
	//setMode(OPENED);
	//addPreviousLS(ls);
	//}
	
	private void backward() {
		if (currentEdition != null) {
			if (currentEdition.backward()) {
				updateInfoPanel();
				//panelForInfo.setNewStructure(currentEdition.getLatexStructure());
			}
		}
	}
	
	private void forward() {
		if (currentEdition != null) {
			if (currentEdition.forward()) {
				updateInfoPanel();
			}
		}
		//System.out.println("forward");
		
	}
	
	private void saveAllAndCompileCurrent() {
		if (currentEdition == null) {
			return;
		}
		
		File file = currentEdition.getFile();
		if (file == null) {
			return;
		}
		
		saveAllProjects();
		
		compile(file);
	}
	
	private void saveCurrentAndCompileCurrent() {
		// Saving document
		
		if (currentEdition == null) {
			return;
		}
		
		File file = currentEdition.getFile();
		if (file == null) {
			return;
		}
		
		if (actions[LEGUIAction.ACT_SAVE].isEnabled()) {
			saveProject();
		}
		
		compile(file);
	}
	
	private void compile() {
		if (currentEdition == null) {
			return;
		}
		
		File file = currentEdition.getFile();
		if (file == null) {
			return;
		}
		compile(file);
	}
	
	private void compile(File file) {
		
		String compileAction;
		if (ConfigurationLatexEditor.CompilationPath == null) {
			compileAction = "latex " + file.getAbsolutePath() ;
		} else {
			compileAction = ConfigurationLatexEditor.CompilationPath +  " " + FileUtils.getDirectoryOf(file);
		}
		
		panelForCompilation.compile(compileAction);
	}
	
	private void makeClean() {
		if (currentEdition == null) {
			return;
		}
		
		File file = currentEdition.getFile();
		if (file == null) {
			return;
		}
		
		String compileAction;
		if (ConfigurationLatexEditor.MakeCleanPath == null) {
			compileAction = "make clean " + file.getAbsolutePath() ;
		} else {
			compileAction = ConfigurationLatexEditor.MakeCleanPath +  " " + FileUtils.getDirectoryOf(file);
		}
		
		panelForCompilation.compile(compileAction);
	}
	
	private void makeUltraclean() {
		if (currentEdition == null) {
			return;
		}
		
		File file = currentEdition.getFile();
		if (file == null) {
			return;
		}
		
		String compileAction;
		if (ConfigurationLatexEditor.MakeUltraCleanPath == null) {
			compileAction = "make ultraclean " + file.getAbsolutePath() ;
		} else {
			compileAction = ConfigurationLatexEditor.MakeUltraCleanPath +  " " + FileUtils.getDirectoryOf(file);
		}
		
		panelForCompilation.compile(compileAction);
	}
	
	private void viewPdf() {
		if (currentEdition == null) {
			return;
		}
		
		File file = currentEdition.getFile();
		if (file == null) {
			return;
		}
		
		viewPdfFile(file);
	}
	
	private void viewPdfNotes() {
		//TraceManager.addDev("View pdf notes");
		if (currentEdition == null) {
			return;
		}
		File file = currentEdition.getFile();
		String name = FileUtils.addBeforeFileExtension(file.getAbsolutePath(), "_notes");
		
        viewPdfFile(new File(name));
	}
	
	private void viewPdfCurrent() {
		TraceManager.addDev("View pdf current");
		if (currentEdition == null) {
			return;
		}
		File file = currentEdition.getFile();
		String name = FileUtils.addBeforeFileExtension(file.getAbsolutePath(), "_current");
		
        viewPdfFile(new File(name));
	}
	
	private void viewPdfFile(File file) {
		
		String action;
		if (ConfigurationLatexEditor.PDFVizualizerPath == null) {
			action = "acroread " + FileUtils.changeFileExtension(file.getAbsolutePath(), "pdf");
		} else {
			action = ConfigurationLatexEditor.PDFVizualizerPath +  " " + FileUtils.changeFileExtension(file.getAbsolutePath(), "pdf");
		}
		
		SimpleProcessThread spt = new SimpleProcessThread(action);
		spt.go();
		//panelForCompilation.execute(action);
	}
	
	private void indent() {
		//System.out.println("Indent");
		/*addPreviousLS(ls);
		selectedBeamerElement.indent();
		panelForEdition.setText(selectedBeamerElement.getText());
		panelForSlides.changeMade();*/
		setStatusBarText("Identing done");
	}
    
    public void aboutVersion() {
        JFrameBasicText jft = new JFrameBasicText("About Latex Editor ...", DefaultText.getAboutText(), IconManager.imgic10, this);
        //jft.setIconImage(IconManager.imgic10);
        jft.setSize(800, 750);
        GraphicLib.centerOnParent(jft);
        jft.setVisible(true);
        
    }
	
    
    
    /*public void screenCapture() {
	//Select file
	File file = selectFileForCapture();
	Toolkit toolkit = Toolkit.getDefaultToolkit();
	Dimension screenSize = toolkit.getScreenSize();
	Rectangle screenRect = new Rectangle(screenSize);
	performScreenCapture(screenRect, file);
    }
    
    public void windowCapture() {
	//Select file
	File file = selectFileForCapture();
	if (file == null)
	return;
	
	Rectangle screenRect = new Rectangle(frame.getLocation().x, frame.getLocation().y, frame.getWidth(), frame.getHeight());
	performScreenCapture(screenRect, file);
    }*/
    
    /*public void diagramCapture() {
	if (tabs.size() < 1) {
	JOptionPane.showMessageDialog(frame,
	"No diagram is under edition",
	"Error",
	JOptionPane.INFORMATION_MESSAGE);
	return;
	}
	
	File file = selectFileForCapture();
	if (file == null)
	return;
	
	TDiagramPanel tdp1 = getCurrentTDiagramPanel();
	if (tdp1 != null) {
	BufferedImage image = tdp1.performMinimalCapture();
	writeImageCapture(image, file, true);
	}
    }*/
    
	
    
    /*public File selectFileForCapture() {
	File file = null;
	int returnVal = jfcimg.showSaveDialog(frame);
	if(returnVal == JFileChooser.APPROVE_OPTION) {
	file = jfcimg.getSelectedFile();
	file = FileUtils.addFileExtensionIfMissing(file, TImgFilter.getExtension());
	
	}
	if(!checkFileForSave(file)) {
	JOptionPane.showMessageDialog(frame,
	"The capture could not be performed: invalid file",
	"Error",
	JOptionPane.INFORMATION_MESSAGE);
	return null;
	}
	return file;
    }
    
    public void performScreenCapture(Rectangle rect, File file) {
	frame.paint(frame.getGraphics());
	try {
	Robot robot = new Robot();
	BufferedImage image = robot.createScreenCapture(rect);
	// save captured image to PNG file
	ImageIO.write(image, "png", file);
	} catch (Exception e) {
	JOptionPane.showMessageDialog(frame,
	"The capture could not be performed:" + e.getMessage(),
	"Error",
	JOptionPane.INFORMATION_MESSAGE);
	return;
	}
	JOptionPane.showMessageDialog(frame,
	"The capture was correctly performed",
	"Screen capture ok",
	JOptionPane.INFORMATION_MESSAGE);
	return;
    }
    
    public boolean writeImageCapture(BufferedImage image, File file, boolean info) {
	frame.paint(frame.getGraphics());
	try {
	// save captured image to PNG file
	ImageIO.write(image, "png", file);
	} catch (Exception e) {
	JOptionPane.showMessageDialog(frame,
	"The capture could not be performed:" + e.getMessage(),
	"Error",
	JOptionPane.INFORMATION_MESSAGE);
	return false;
	}
	if (info) {
	JOptionPane.showMessageDialog(frame,
	"The capture was correctly performed",
	"Screen capture ok",
	JOptionPane.INFORMATION_MESSAGE);
	}
	return true;
    }*/
	
	
    
	/* public int getTypeButtonSelected() {
	return typeButtonSelected;
    }*/
    
    /*public void actionOnButton(int type, int id) {
	//typeButtonSelected = type;
	//idButtonSelected = id;
	//TDiagramPanel tdp1 = ((TURTLEPanel)(tabs.elementAt(mainTabbedPane.getSelectedIndex()))).tdp;
	//TDiagramPanel tdp1 = getCurrentTDiagramPanel();
	//System.out.println("Selected TDiagramPanel=" + tdp1.getName());
	//tdp1.repaint();
    }*/
    
    /*public int getIdButtonSelected() {
	return idButtonSelected;
    }*/
    
	
	
    
    
    
    public void windowClosing(WindowEvent e) {
        //frame.setVisible(false);
        quitApplication();
    }
    
    public void windowClosed(WindowEvent e) {
    }
    
    public void windowOpened(WindowEvent e) {
    }
    
    public void windowIconified(WindowEvent e) {
    }
    
    public void windowDeiconified(WindowEvent e) {
    }
    
    public void windowActivated(WindowEvent e) {
    }
    
    public void windowDeactivated(WindowEvent e) {
    }
    
    
    // LEGUIActions Event listener
    
    public void keyTyped(KeyEvent e) {
        TraceManager.addDev("KEY TYPED");
    }
    
    public void keyPressed(KeyEvent e) {
        TraceManager.addDev("KEY PRESSED: ");
    }
    
    public void keyReleased(KeyEvent e) {
        TraceManager.addDev("KEY RELEASED: ");
    }
	
	public void command(int _index) {
		if (currentEdition != null) {
			currentEdition.command(_index);
		}
	}
    
    
    public void	actionPerformed(ActionEvent evt)  {
        String command = evt.getActionCommand();
        TraceManager.addDev("Command:" + command);
        
        // Compare the action command to the known actions.
        if (command.equals(actions[LEGUIAction.ACT_NEW].getActionCommand()))  {
            newProject();
        } else if (command.equals(actions[LEGUIAction.ACT_OPEN].getActionCommand())) {
            openProject();
        } else if (command.equals(actions[LEGUIAction.ACT_LAST_OPEN].getActionCommand())) {
            openLastProject();
        } else if (command.equals(actions[LEGUIAction.ACT_SAVE].getActionCommand())) {
            saveProject();
        } else if (command.equals(actions[LEGUIAction.GENERATE_NOTES].getActionCommand())) {
            generateNotes();
		} else if (command.equals(actions[LEGUIAction.GENERATE_CURRENT].getActionCommand())) {
            generateCurrent();
        } else if (command.equals(actions[LEGUIAction.ACT_SAVE].getActionCommand())) {
            closeCurrentProject();
        } else if (command.equals(actions[LEGUIAction.ACT_SAVE_AS].getActionCommand())) {
            saveAsProject();
        } else if (command.equals(actions[LEGUIAction.ACT_SAVE_ALL].getActionCommand())) {
            saveAllProjects();
        } else if (command.equals(actions[LEGUIAction.ACT_QUIT].getActionCommand())) {
            quitApplication();
        } else if (command.equals(actions[LEGUIAction.ACT_BACKWARD].getActionCommand())) {
            backward();
        } else if (command.equals(actions[LEGUIAction.ACT_FORWARD].getActionCommand())) {
            forward();
		} else if (command.equals(actions[LEGUIAction.ACT_INDENT].getActionCommand())) {
            indent();
		} else if (command.equals(actions[LEGUIAction.ABOUT].getActionCommand())) {
            aboutVersion();
		} else if (command.equals(actions[LEGUIAction.ACT_COMPILE_ONLY].getActionCommand())) {
            compile();
        } else if (command.equals(actions[LEGUIAction.ACT_COMPILE].getActionCommand())) {
            saveCurrentAndCompileCurrent();
        } else if (command.equals(actions[LEGUIAction.ACT_COMPILE_ALL].getActionCommand())) {
            saveAllAndCompileCurrent();	
		} else if (command.equals(actions[LEGUIAction.ACT_MAKE_CLEAN].getActionCommand())) {
            makeClean();	
		} else if (command.equals(actions[LEGUIAction.ACT_MAKE_ULTRACLEAN].getActionCommand())) {
            makeUltraclean();
		} else if (command.equals(actions[LEGUIAction.ACT_VIEW_PDF].getActionCommand())) {
            viewPdf();
		} else if (command.equals(actions[LEGUIAction.ACT_VIEW_PDF_NOTES].getActionCommand())) {
            viewPdfNotes();
		} else if (command.equals(actions[LEGUIAction.ACT_VIEW_PDF_CURRENT].getActionCommand())) {
            viewPdfCurrent();
			
			// Latex commands
        } else if (command.equals(actions[LEGUIAction.INSERT_ITEMIZE].getActionCommand())) {
			command(LEGUIAction.INSERT_ITEMIZE);
        } else if (command.equals(actions[LEGUIAction.INSERT_ENUMERATE].getActionCommand())) {
            command(LEGUIAction.INSERT_ENUMERATE);
        } else if (command.equals(actions[LEGUIAction.INSERT_ITEM].getActionCommand())) {
            command(LEGUIAction.INSERT_ITEM);
        }  else if (command.equals(actions[LEGUIAction.ALIGN_LEFT].getActionCommand())) {
        	command(LEGUIAction.ALIGN_LEFT);
        } else if (command.equals(actions[LEGUIAction.ALIGN_RIGHT].getActionCommand())) {
            command(LEGUIAction.ALIGN_RIGHT);
        } else if (command.equals(actions[LEGUIAction.ALIGN_CENTER].getActionCommand())) {
            command(LEGUIAction.ALIGN_CENTER);
        } else if (command.equals(actions[LEGUIAction.ALIGN_JUSTIFY].getActionCommand())) {
            command(LEGUIAction.ALIGN_JUSTIFY);
        } else if (command.equals(actions[LEGUIAction.TEXT_BOLD].getActionCommand())) {
            command(LEGUIAction.TEXT_BOLD);
        } else if (command.equals(actions[LEGUIAction.TEXT_ITALICS].getActionCommand())) {
            command(LEGUIAction.TEXT_ITALICS);
        } else if (command.equals(actions[LEGUIAction.TEXT_UNDERLINE].getActionCommand())) {
            command(LEGUIAction.TEXT_UNDERLINE);
        } else if (command.equals(actions[LEGUIAction.INSERT_FIGURE].getActionCommand())) {
        	command(LEGUIAction.INSERT_FIGURE);
			
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_0].getActionCommand())) {
            command(LEGUIAction.TEXT_SIZE_0);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_1].getActionCommand())) {
            command(LEGUIAction.TEXT_SIZE_1);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_2].getActionCommand())) {
        	command(LEGUIAction.TEXT_SIZE_2);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_3].getActionCommand())) {
        	command(LEGUIAction.TEXT_SIZE_3);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_4].getActionCommand())) {
        	command(LEGUIAction.TEXT_SIZE_4);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_5].getActionCommand())) {
        	command(LEGUIAction.TEXT_SIZE_5);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_6].getActionCommand())) {
            command(LEGUIAction.TEXT_SIZE_6);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_7].getActionCommand())) {
        	command(LEGUIAction.TEXT_SIZE_7);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_8].getActionCommand())) {
        	command(LEGUIAction.TEXT_SIZE_8);
        } else if (command.equals(actions[LEGUIAction.TEXT_SIZE_9].getActionCommand())) {
            command(LEGUIAction.TEXT_SIZE_9);
			
			
		} else if (command.equals(actions[LEGUIAction.INSERT_TEMPLATE].getActionCommand())) {
            command(LEGUIAction.INSERT_TEMPLATE);
		} else if (command.equals(actions[LEGUIAction.INSERT_REFERENCE].getActionCommand())) {
            command(LEGUIAction.INSERT_REFERENCE);
		} else if (command.equals(actions[LEGUIAction.INSERT_CITE].getActionCommand())) {
            command(LEGUIAction.INSERT_CITE);
		} else if (command.equals(actions[LEGUIAction.INSERT_LABEL].getActionCommand())) {
            command(LEGUIAction.INSERT_LABEL);
		} else if (command.equals(actions[LEGUIAction.INSERT_SECTION].getActionCommand())) {
            command(LEGUIAction.INSERT_SECTION);
		} else if (command.equals(actions[LEGUIAction.INSERT_SUBSECTION].getActionCommand())) {
            command(LEGUIAction.INSERT_SUBSECTION);
		} else if (command.equals(actions[LEGUIAction.INSERT_BLOCK].getActionCommand())) {
            command(LEGUIAction.INSERT_BLOCK);
		} else if (command.equals(actions[LEGUIAction.INSERT_ALERTBLOCK].getActionCommand())) {
            command(LEGUIAction.INSERT_ALERTBLOCK);
		} else if (command.equals(actions[LEGUIAction.INSERT_EXAMPLEBLOCK].getActionCommand())) {
            command(LEGUIAction.INSERT_EXAMPLEBLOCK);
		} else if (command.equals(actions[LEGUIAction.INSERT_NOTE].getActionCommand())) {
            command(LEGUIAction.INSERT_NOTE);
		} else if (command.equals(actions[LEGUIAction.INSERT_COLUMNS].getActionCommand())) {
            command(LEGUIAction.INSERT_COLUMNS);
		} else if (command.equals(actions[LEGUIAction.INSERT_COLUMN].getActionCommand())) {
            command(LEGUIAction.INSERT_COLUMN);
		} else if (command.equals(actions[LEGUIAction.INSERT_LISTING].getActionCommand())) {
            command(LEGUIAction.INSERT_LISTING);
		} else if (command.equals(actions[LEGUIAction.INSERT_TABULAR].getActionCommand())) {
            command(LEGUIAction.INSERT_TABULAR);
		} else if (command.equals(actions[LEGUIAction.INSERT_ROWCOLOR].getActionCommand())) {
            command(LEGUIAction.INSERT_ROWCOLOR);
		} else if (command.equals(actions[LEGUIAction.INSERT_HLINE].getActionCommand())) {
            command(LEGUIAction.INSERT_HLINE);
		} else if (command.equals(actions[LEGUIAction.INSERT_FRAME].getActionCommand())) {
            command(LEGUIAction.INSERT_FRAME);
		} else if (command.equals(actions[LEGUIAction.FIND].getActionCommand())) {
            applyFind();
		} else if (command.equals(actions[LEGUIAction.FIND_PREVIOUS].getActionCommand())) {
            findPrevious();
		} else if (command.equals(actions[LEGUIAction.FIND_FRAME].getActionCommand())) {
            findFrame();
		} else if (command.equals(actions[LEGUIAction.REPLACE_ONE].getActionCommand())) {
            replaceOne();
		} else if (command.equals(actions[LEGUIAction.REPLACE_ALL].getActionCommand())) {
            replaceAll();
        }
    }
	
	public void paneAction(ChangeEvent e) {
        TraceManager.addDev("Pane action");
        try {
            currentEdition = editions.get(mainTabbedPane.getSelectedIndex());
			if (currentEdition.getFile() != null) {
				setTitle(currentEdition.getFile().getAbsolutePath());
			}
			updatePanels();
			manageFind(find);
        } catch	(Exception ex) {
            TraceManager.addDev("Exception pane action: " + ex.getMessage());
			setTitle("No opened file");
			updatePanels();
        }
    }
	
	/**
	* This adapter is constructed to handle mouse over	component events.
	*/
    private class MouseHandler extends MouseAdapter  {
        
        private	JLabel label;
        
        /**
		* ctor	for the	adapter.
		* @param label	the JLabel which will recieve value of the
		*		Action.LONG_DESCRIPTION	key.
		*/
        public MouseHandler(JLabel label)  {
            setLabel(label);
        }
        
        public void setLabel(JLabel label)  {
            this.label = label;
        }
        
        public void mouseEntered(MouseEvent evt)  {
            if (evt.getSource()	instanceof AbstractButton)  {
                AbstractButton button =	(AbstractButton)evt.getSource();
                Action action =	button.getAction(); // getAction is new	in JDK 1.3
                if (action != null)  {
                    String message = (String)action.getValue(Action.LONG_DESCRIPTION);
                    label.setText(message);
                }
            }
        }
    }
	
	public static String stringToLatex(String _input) {
		if (_input != null) {
			_input = Conversion.replaceAllChar(_input, '_', "\\_");
			_input = Conversion.replaceAllString(_input, "->", "$rightarrow$");
		}
		return _input;
	}
	
	public String removeantishlashr(String _s) {
		String ret = _s.replaceAll("\r", "");
		return ret;
	}
    
} // Class MainGUI
